// background.js - Service Worker

let roomEmotelists = {}; // Store lists per room: { "roomName": { ...emotes... } }
console.log("Background script starting up...");

// Helper to get room name from URL
function getRoomNameFromUrl(urlString) {
    if (!urlString) return null;
    try {
        const url = new URL(urlString);
        if (url.hostname === 'dab.chat') {
            const pathParts = url.pathname.split('/');
            // Check for paths like /room/roomname
            if (pathParts.length >= 3 && pathParts[1] === 'room' && pathParts[2]) {
                return pathParts[2]; // Return the room name
            }
        }
    } catch (e) {
        console.error("Error parsing URL in background:", urlString, e);
    }
    return null; // Not a valid room URL or parsing error
}


// Main message listener
chrome.runtime.onMessage.addListener((request, sender, sendResponse) => {
    console.log("Background received message:", request.action, "from tab:", sender.tab?.id);

    let roomName = null;
    if (sender.tab && sender.tab.url) {
        roomName = getRoomNameFromUrl(sender.tab.url);
    }
    // Allow request to override roomName if provided (e.g., for getEmotelist)
    if (request.room) {
        roomName = request.room;
    }

    console.log(`Background: Determined Room Name: ${roomName} for action: ${request.action}`);

    // --- Handler to provide the currently cached emotelist for a specific room ---
    if (request.action === "getEmotelist") {
        if (!roomName) {
             console.warn("Background: 'getEmotelist' called without a valid room context.");
             sendResponse({}); // Send empty object if no room context
             return false;
        }
        const listForRoom = roomEmotelists[roomName] || {};
        const count = Object.keys(listForRoom).length;
        console.log(`Background: Action 'getEmotelist' for room '${roomName}'. Sending cached list with ${count} items.`);
        try { sendResponse(structuredClone(listForRoom)); }
        catch (e) { sendResponse(JSON.parse(JSON.stringify(listForRoom))); } // Fallback clone
        return false; // Synchronous response
    }

    // --- Handler to update the emotelist (Now requires room context, usually from injectScript) ---
    // Note: This action might become less relevant if injection is the primary update mechanism.
    else if (request.action === "updateEmotelist") {
        if (!roomName) {
            console.error("Background: Action 'updateEmotelist' received without room context.");
            sendResponse({ success: false, error: "Missing room context" });
            return false;
        }
        if (request.emotelist && typeof request.emotelist === 'object') {
            roomEmotelists[roomName] = request.emotelist;
            const newCount = Object.keys(roomEmotelists[roomName]).length;
            console.log(`Background: Action 'updateEmotelist' for room '${roomName}'. Cache updated. New: ${newCount} entries.`);
            sendResponse({ success: true, count: newCount });
        } else {
            console.error(`Background: Action 'updateEmotelist' for room '${roomName}' received invalid data:`, request.emotelist);
            sendResponse({ success: false, error: "Invalid emotelist data" });
        }
        return false; // Synchronous response
    }

    // --- Handler to perform script injection on request from content script ---
    else if (request.action === "injectScript") {
        if (!(sender && sender.tab && sender.tab.id && sender.tab.url)) {
            console.error("Background: Action 'injectScript' cannot proceed, sender tab ID or URL is missing.");
            sendResponse({ success: false, error: "Missing sender tab ID/URL" });
            return false; // Synchronous error response
        }
        if (!roomName) {
             console.error(`Background: Action 'injectScript' cannot determine room name from URL: ${sender.tab.url}. Aborting injection.`);
             sendResponse({ success: false, error: "Cannot determine room name from URL" });
             return false; // Synchronous error response
        }

        const targetTabId = sender.tab.id;
        console.log(`Background: Action 'injectScript' for room '${roomName}' in tab ${targetTabId}. Initiating executeScript.`);

        // Use executeScript which returns a Promise
        chrome.scripting.executeScript({
            target: { tabId: targetTabId },
            func: () => { // Injection function remains the same
                    console.log("[Injected Script] Running (manual object build version)...");
                    let plainEmoteObject = {}; let errorMsg = null;
                    if (typeof window.emotelist === 'object' && window.emotelist !== null) {
                        const sourceEmotelist = window.emotelist;
                        console.log(`[Injected Script] Found window.emotelist. Type: ${typeof sourceEmotelist}`);
                        try {
                            let copiedCount = 0;
                            for (const key in sourceEmotelist) {
                                if (Object.prototype.hasOwnProperty.call(sourceEmotelist, key) &&
                                    typeof key === 'string' && typeof sourceEmotelist[key] === 'string') {
                                    plainEmoteObject[key] = sourceEmotelist[key]; copiedCount++;
                                } // else skip non-string keys/values
                            }
                            console.log(`[Injected Script] Copied ${copiedCount} key-value pairs.`);
                        } catch (e) { errorMsg = `Error copying properties: ${e.message}`; plainEmoteObject = { _error: errorMsg }; }
                    } else { errorMsg = "window.emotelist not found or not object."; plainEmoteObject = { _error: errorMsg }; }
                    console.log("[Injected Script] Final plain object being returned:", plainEmoteObject);
                    return plainEmoteObject; // Return the plain object
                },
            world: "MAIN" // Execute in the page's context to access window.emotelist
        })
        .then(results => {
            const injectionResultObject = results && results.length > 0 ? results[0].result : null;
            console.log(`Background: executeScript Promise resolved for room '${roomName}'. Received object:`, injectionResultObject);

            if (injectionResultObject && typeof injectionResultObject === 'object' && !injectionResultObject._error) {
                const count = Object.keys(injectionResultObject).length;
                console.log(`Background: Injection for room '${roomName}' successful. Found ${count} emotes. Updating cache.`);
                roomEmotelists[roomName] = injectionResultObject; // *** Store list associated with roomName ***
                console.log(`Background: Calling sendResponse (SUCCESS) for room '${roomName}' with emotelist:`, roomEmotelists[roomName]);
                // Send the fetched list back to the specific content script that requested it
                sendResponse({ success: true, emotelist: roomEmotelists[roomName] });
            } else {
                let errorMsg = "Injected script returned null, invalid object, or error marker.";
                 if (injectionResultObject && injectionResultObject._error) { errorMsg = injectionResultObject._error; }
                 else if (results && results.length > 0 && results[0].error) { errorMsg = `Injection execution failed: ${results[0].error}`; }
                 else if (injectionResultObject === null) { errorMsg = "Received null result from injection."; }
                 else { errorMsg = `Received unexpected/invalid result type: ${typeof injectionResultObject}`; }
                console.warn(`Background: Script injection for room '${roomName}' failed. Reason: ${errorMsg}. Calling sendResponse (FAILURE).`);
                // Still clear the list for this room on failure? Maybe not, keep old one? Let's keep the old one for now.
                // roomEmotelists[roomName] = {}; // Optional: Clear cache on failure
                sendResponse({ success: false, error: errorMsg });
            }
            // No return needed inside .then()
        })
        .catch(error => {
            console.error(`Background: Error DURING executeScript call for room '${roomName}', tab ${targetTabId}:`, error);
            sendResponse({ success: false, error: `Failed during executeScript call: ${error.message}` });
            // No return needed inside .catch()
        });

        // CRITICAL: Return true HERE to keep message channel open for async response
        console.log(`Background: Returning true for action 'injectScript' to keep message channel open.`);
        return true;
    }
    // --- End Inject Script Handler ---

    else {
        console.log("Background: Received unknown action:", request.action);
        // No response needed, return false/undefined implicitly
        return false;
    }
}); // End of chrome.runtime.onMessage.addListener


chrome.runtime.onInstalled.addListener(details => {
    console.log("Extension installed or updated:", details.reason);
    // Could potentially clear all stored emotelists on update if needed
    // roomEmotelists = {};
});

console.log("Background script finished setup.");