// content.js

// Store a local copy of the emotelist specific to this content script instance/page
let localEmotelist = {};
let isFetchingEmotelist = false; // Prevent simultaneous fetches for this page
let currentRoomName = null; // Store the room name for this page
let keyupDebounceTimer = null; // Timer for debouncing keyup events
const KEYUP_DEBOUNCE_MS = 150; // Debounce delay in milliseconds
const MAX_SUGGESTIONS = 15; // Max suggestions to display *conditionally*
let currentSuggestionIndex = -1; // Index of the currently highlighted suggestion (START UNSELECTED)
let showEmotePreviews = true; // Default value, will be updated from storage

// --- NEW: Variables for DELAYED Emote Preview ---
let emotePreviewPopupElement = null;
let emotePreviewTimerId = null; // Stores the setTimeout ID
const EMOTE_PREVIEW_DELAY = 1500; // Delay in ms (1.5 seconds)

// --- Get current room name from URL ---
function getCurrentRoomName() {
    try {
        const pathParts = window.location.pathname.split('/');
        if (pathParts.length >= 3 && pathParts[1] === 'room' && pathParts[2]) {
            return pathParts[2];
        }
    } catch (e) {
        console.error("Error getting room name from path:", e);
    }
    return null;
}

// --- Function to load settings ---
async function loadSettings() {
    try {
        const result = await chrome.storage.sync.get(['showEmotePreviews']);
        showEmotePreviews = result.showEmotePreviews !== false;
    } catch (error) {
        console.error(`[${currentRoomName || 'Global'}] Error loading settings:`, error);
        showEmotePreviews = true; // Fallback
    }
}


// --- Fetch Emotelist via Injection ---
async function fetchEmotelistViaInjection() {
    if (isFetchingEmotelist) { return false; }
    if (!currentRoomName) { console.warn("GIF Suggester: Cannot fetch emotelist, not in a recognized room."); return false; }
    isFetchingEmotelist = true;
    let success = false;
    try {
        const response = await chrome.runtime.sendMessage({ action: "injectScript" });
        if (chrome.runtime.lastError) {
            console.error(`GIF Suggester: [${currentRoomName}] Error during injectScript message:`, chrome.runtime.lastError.message);
        } else if (response?.success && response.emotelist && typeof response.emotelist === 'object') {
            const count = Object.keys(response.emotelist).length;
            console.log(`GIF Suggester: [${currentRoomName}] SUCCESS - Received ${count} emotes.`);
            localEmotelist = response.emotelist;
            success = true;
        } else {
             let failureReason = response?.error || "Unknown (check background logs)";
             console.warn(`GIF Suggester: [${currentRoomName}] Background script indicated injection/fetch failed. Reason: ${failureReason}.`);
        }
    } catch (error) {
        console.error(`GIF Suggester: [${currentRoomName}] Exception sending/receiving injectScript message:`, error);
    } finally {
        isFetchingEmotelist = false;
    }
    return success;
}


// --- Get Suggestions Function (Improved Sorting) ---
async function getGifSuggestions(prefix) {
    if (!currentRoomName) { return []; }

    let currentEmotelist = localEmotelist;
    if (Object.keys(currentEmotelist).length === 0 && !isFetchingEmotelist) {
        isFetchingEmotelist = true;
        try {
            const response = await chrome.runtime.sendMessage({ action: "getEmotelist", room: currentRoomName });
            if (response && Object.keys(response).length > 0) { localEmotelist = response; currentEmotelist = localEmotelist; }
            else { currentEmotelist = {}; }
        } catch (error) { console.warn(`[${currentRoomName}] Exception fetching cached list:`, error); currentEmotelist = {}; }
        finally { isFetchingEmotelist = false; }
    } else if (isFetchingEmotelist) { return []; }

    const allKeys = Object.keys(currentEmotelist);
    if (allKeys.length === 0) { return []; }

    const searchTerm = prefix.substring(1).toLowerCase();
    if (!searchTerm) return [];

    const startsWithMatches = [];
    const includesMatches = [];
    allKeys.forEach(key => {
        const keyLower = key.toLowerCase();
        if (keyLower.startsWith(searchTerm)) { startsWithMatches.push(key); }
        else if (keyLower.includes(searchTerm)) { includesMatches.push(key); }
    });
    startsWithMatches.sort((a, b) => a.localeCompare(b, undefined, { sensitivity: 'base' }));
    includesMatches.sort((a, b) => a.localeCompare(b, undefined, { sensitivity: 'base' }));
    const finalSortedKeys = [...startsWithMatches, ...includesMatches];
    return finalSortedKeys.map(key => `:${key}`);
}

// --- NEW: Create or Get Delayed Emote Preview Element ---
function ensureEmotePreviewElement() {
    if (!emotePreviewPopupElement || !document.body.contains(emotePreviewPopupElement)) {
        emotePreviewPopupElement = document.createElement('div');
        emotePreviewPopupElement.id = 'emote-preview-popup'; // Use new ID from CSS
        emotePreviewPopupElement.innerHTML = '<img>'; // Add img tag inside
        // Add directly to body, positioning will handle placement
        document.body.appendChild(emotePreviewPopupElement);
        console.log("DEBUG: Created delayed emote preview element.");
    }
    return emotePreviewPopupElement;
}

// --- NEW: Show Delayed Emote Preview Logic ---
function showEmotePreview(emoteKey) {
    const suggestionPopup = document.getElementById('gif-suggestions-popup');
    const suggestions = suggestionPopup ? Array.from(suggestionPopup.children) : [];

    // Ensure the suggestion popup is still visible and the index is valid
    if (!suggestionPopup || suggestionPopup.style.display === 'none' || currentSuggestionIndex < 0 || currentSuggestionIndex >= suggestions.length) {
        hideEmotePreview(); // Should already be hidden, but ensure state consistency
        return;
    }

    const highlightedItem = suggestions[currentSuggestionIndex];
    const actualEmoteKey = highlightedItem.getAttribute('data-display-value')?.substring(1);

    // Final check: Is the key we were asked to preview still the one highlighted?
    // This prevents race conditions if the highlight changed just before the timer fired.
    if (!actualEmoteKey || actualEmoteKey !== emoteKey) {
        console.log("DEBUG: Preview aborted, highlight changed before timer fired.");
        hideEmotePreview();
        return;
    }

    const imgSrc = localEmotelist[emoteKey];
    if (!imgSrc || !(imgSrc.startsWith('http:') || imgSrc.startsWith('https:') || imgSrc.startsWith('data:'))) {
        console.warn("DEBUG: No valid image source found for preview:", emoteKey);
        hideEmotePreview(); // Hide if no valid source
        return;
    }

    const previewDiv = ensureEmotePreviewElement();
    const previewImg = previewDiv.querySelector('img');
    if (!previewImg) { return; } // Should not happen

    previewImg.src = imgSrc;
    previewImg.alt = emoteKey;

    // --- Positioning Logic for Delayed Preview ---
    const suggestionRect = suggestionPopup.getBoundingClientRect();
    const itemRect = highlightedItem.getBoundingClientRect(); // Use highlighted item's rect
    const viewportWidth = window.innerWidth;
    const viewportHeight = window.innerHeight;
    const scrollX = window.scrollX;
    const scrollY = window.scrollY;
    const gap = 10; // Gap between suggestion list and preview

    // Make visible temporarily to measure natural dimensions *after* image loads?
    // This is tricky. We might have to guess or set fixed max sizes. Let's use CSS max sizes.
    // We'll position based on potential max size.
    previewDiv.style.visibility = 'hidden'; // Keep hidden while positioning
    previewDiv.classList.add('visible'); // Add class to apply styles (incl max-height)

    const previewWidth = previewDiv.offsetWidth; // Measure after styles applied
    const previewHeight = previewDiv.offsetHeight;

    let previewLeft, previewTop;

    // Calculate available space
    const spaceRight = viewportWidth - suggestionRect.right - gap;
    const spaceLeft = suggestionRect.left - gap;

    // Prefer right if enough space, else left, else fallback (align right edge)
    if (spaceRight >= previewWidth) {
         previewLeft = suggestionRect.right + scrollX + gap;
         console.log("DEBUG: Emote preview placed to the RIGHT.");
    } else if (spaceLeft >= previewWidth) {
        previewLeft = suggestionRect.left + scrollX - previewWidth - gap;
        console.log("DEBUG: Emote preview placed to the LEFT.");
    } else {
        // Fallback: Try aligning right edge of preview with right edge of suggestions
        previewLeft = suggestionRect.right + scrollX - previewWidth;
        console.log("DEBUG: Emote preview placed aligned RIGHT (fallback).");
        // Ensure it doesn't go off left edge
        previewLeft = Math.max(scrollX + gap, previewLeft);
    }

    // Align vertically with the highlighted item, constrained by viewport
    previewTop = itemRect.top + scrollY; // Align top with highlighted item's top
    previewTop = Math.max(scrollY + gap, previewTop); // Don't go off viewport top
    // Ensure bottom doesn't go off viewport bottom
    previewTop = Math.min(scrollY + viewportHeight - previewHeight - gap, previewTop);

    previewDiv.style.left = `${previewLeft}px`;
    previewDiv.style.top = `${previewTop}px`;

    // Make it smoothly visible using CSS transition defined in .visible class
    previewDiv.style.visibility = 'visible'; // Make visible *after* positioning
    console.log(`DEBUG: Showing emote preview for ${emoteKey}`);
}

// --- NEW: Hide Delayed Emote Preview Logic ---
function hideEmotePreview() {
    // Clear any pending timer to show the preview
    if (emotePreviewTimerId) {
        clearTimeout(emotePreviewTimerId);
        emotePreviewTimerId = null;
        // console.log("DEBUG: Cleared emote preview timer.");
    }
    // Hide the preview element itself
    if (emotePreviewPopupElement) {
        emotePreviewPopupElement.classList.remove('visible');
        // Optional: Could set display:none after transition, but opacity=0 usually sufficient
        // console.log("DEBUG: Hiding emote preview element.");
    }
}

// --- Event Listeners Setup ---
window.addEventListener('load', async function() {
    currentRoomName = getCurrentRoomName();
    console.log(`GIF Suggester: Page loaded. Initializing for room: ${currentRoomName || 'Global/Unknown'}`);
    setupTextareaListeners();
    await loadSettings();

    if (currentRoomName) {
        console.log(`GIF Suggester: [${currentRoomName}] Ready. Attempting automatic emote fetch...`);
        setTimeout(fetchEmotelistViaInjection, 1500);
    } else {
        console.warn("GIF Suggester: Not in a recognized Dab.Chat room, automatic fetch skipped.");
    }

    chrome.runtime.onMessage.addListener((request, sender, sendResponse) => {
        if (request.action === "previewSettingChanged") {
            showEmotePreviews = request.showPreviews;
            sendResponse({ success: true });
            // If suggestions are currently visible, re-render them? Or just affect next time.
            // Let's keep it simple and affect next time.
            return false;
        }
         return false;
    });
});

// --- Keyup Listener (with Debounce) ---
document.addEventListener('keyup', function (e) {
    const targetTag = e.target.tagName.toLowerCase();
    const isEditableField = targetTag === 'textarea' || (targetTag === 'input' && e.target.type === 'text');

    if (isEditableField) {
        const ignoredKeys = ['Shift', 'Control', 'Alt', 'Meta', 'ArrowDown', 'ArrowUp', 'ArrowLeft', 'ArrowRight', 'Enter', 'Tab', 'Escape'];
        if (ignoredKeys.includes(e.key)) {
            clearTimeout(keyupDebounceTimer);
            // Don't hide preview here, keydown handles interaction cleanup
            return;
        }

        const element = e.target;
        clearTimeout(keyupDebounceTimer);
        keyupDebounceTimer = setTimeout(async () => {
            const value = element.value || element.textContent || "";
            const cursorPos = element.selectionEnd ?? value.length;
            let wordStart = value.lastIndexOf(' ', cursorPos - 1);
            wordStart = (wordStart === -1 && cursorPos > 0) ? 0 : wordStart + 1;
            const potentialPrefix = value.substring(wordStart, cursorPos);

            if (potentialPrefix.startsWith(':') && potentialPrefix.length > 1) {
                const currentWord = potentialPrefix;
                const suggestions = await getGifSuggestions(currentWord);
                displaySuggestions(suggestions, element, currentWord);
            } else {
                removeSuggestionsPopup(); // This calls hideEmotePreview
            }
        }, KEYUP_DEBOUNCE_MS);
    } else {
        removeSuggestionsPopup(); // This calls hideEmotePreview
    }
});

// --- Setup Textarea Listeners (via MutationObserver) ---
function setupTextareaListeners() {
    let debounceTimer;
    const addListenersToTextareas = () => {
        document.querySelectorAll('textarea:not([data-gif-suggest-listener]), input[type="text"]:not([data-gif-suggest-listener])').forEach(element => {
            element.addEventListener('keydown', handleKeydown, true); // Capture=true is crucial
            element.setAttribute('data-gif-suggest-listener', 'true');
        });
    };
    const observer = new MutationObserver(function(mutations) {
        let needsListenerUpdate = false;
        mutations.forEach(function(mutation) {
            if (mutation.addedNodes?.length > 0) {
                 mutation.addedNodes.forEach(node => {
                    if (node.nodeType === 1) {
                        const selector = 'textarea:not([data-gif-suggest-listener]), input[type="text"]:not([data-gif-suggest-listener])';
                        if (node.matches?.(selector) || node.querySelector?.(selector)) { needsListenerUpdate = true; }
                    }
                 });
            }
        });
        if (needsListenerUpdate) { clearTimeout(debounceTimer); debounceTimer = setTimeout(addListenersToTextareas, 150); }
    });
    observer.observe(document.body, { childList: true, subtree: true });
    addListenersToTextareas(); // Initial scan
    console.log("GIF Suggester: Initial editable element listener setup complete.");
}

// --- Handle Keydown for Navigation/Selection ---
function handleKeydown(e) {
    const popup = document.getElementById('gif-suggestions-popup');
    if (!popup || popup.style.display === 'none' || !popup.hasChildNodes()) { return; }
    const suggestions = Array.from(popup.children);
    if (suggestions.length === 0) { return; }

    let handled = false;

    // Hide preview immediately on any interaction via keyboard that might change state
     hideEmotePreview(); // Clear existing timers/preview on keydown

    if (e.key === 'Escape') {
        removeSuggestionsPopup(); // Also calls hideEmotePreview
        handled = true;
    } else if (e.key === 'Enter') {
        if (currentSuggestionIndex >= 0 && currentSuggestionIndex < suggestions.length) {
            suggestions[currentSuggestionIndex].click(); // Calls removeSuggestionsPopup -> hideEmotePreview
            handled = true;
        } else {
            removeSuggestionsPopup(); // Calls hideEmotePreview
            // handled remains false, allowing default Enter action
        }
    } else if (e.key === 'Tab') {
        if (currentSuggestionIndex >= 0 && currentSuggestionIndex < suggestions.length) {
            suggestions[currentSuggestionIndex].click(); // Calls removeSuggestionsPopup -> hideEmotePreview
        } else {
            removeSuggestionsPopup(); // Calls hideEmotePreview
        }
        handled = true;
    } else if (e.key === 'ArrowDown') {
        let nextIndex;
        if (currentSuggestionIndex < 0) { nextIndex = 0; }
        else { nextIndex = (currentSuggestionIndex + 1) % suggestions.length; }
        if (nextIndex !== currentSuggestionIndex) {
             currentSuggestionIndex = nextIndex;
             highlightSuggestion(suggestions, currentSuggestionIndex); // Will start new timer
        }
        handled = true;
    } else if (e.key === 'ArrowUp') {
        let prevIndex;
        if (currentSuggestionIndex < 0) { prevIndex = suggestions.length - 1; }
        else { prevIndex = (currentSuggestionIndex - 1 + suggestions.length) % suggestions.length; }
        if (prevIndex !== currentSuggestionIndex) {
            currentSuggestionIndex = prevIndex;
            highlightSuggestion(suggestions, currentSuggestionIndex); // Will start new timer
        }
        handled = true;
    } else if (e.key === 'ArrowLeft' || e.key === 'ArrowRight') {
        if (currentSuggestionIndex >= 0) {
            // Arrow L/R should potentially keep the preview visible? Or hide?
            // Let's hide it for consistency, as highlight hasn't changed but user is interacting.
             hideEmotePreview(); // Explicitly hide/cancel timer on L/R if item selected
            handled = true;
        }
    }

    if (handled) {
        e.preventDefault();
        e.stopPropagation();
    }
}

// --- MODIFIED: Highlight Suggestion Item (Starts Preview Timer) ---
function highlightSuggestion(suggestions, index) {
    // 1. Clear any existing preview/timer immediately
    hideEmotePreview();

    suggestions.forEach((child, i) => {
        const shouldHighlight = (i === index);
        if (child.classList.contains('highlighted') !== shouldHighlight) {
            child.classList.toggle('highlighted', shouldHighlight);
        }
        if (shouldHighlight) {
            child.scrollIntoView({ behavior: 'auto', block: 'nearest' });
        }
    });

    // 2. If a valid item is highlighted, start the timer to show its preview
    if (index >= 0 && index < suggestions.length) {
        const highlightedItem = suggestions[index];
        const emoteKey = highlightedItem.getAttribute('data-display-value')?.substring(1);
        if (emoteKey) {
            // Start the timer
            emotePreviewTimerId = setTimeout(() => {
                // Check *again* before showing, in case state changed during the delay
                if (currentSuggestionIndex === index && document.body.contains(highlightedItem)) {
                     showEmotePreview(emoteKey);
                } else {
                    console.log("DEBUG: Preview timer fired, but state changed. Aborting show.");
                }
                emotePreviewTimerId = null; // Clear timer ID after it runs
            }, EMOTE_PREVIEW_DELAY);
            // console.log(`DEBUG: Started emote preview timer (${EMOTE_PREVIEW_DELAY}ms) for index ${index} (${emoteKey})`);
        }
    }
}

// --- Display Suggestions Popup ---
function displaySuggestions(suggestions, targetElement, currentWord) {
  let popup = document.getElementById('gif-suggestions-popup');
  const popupExisted = !!popup;
  let suggestionsToDisplay = suggestions;

  if (currentWord && currentWord.length === 2 && suggestions.length > MAX_SUGGESTIONS) {
      suggestionsToDisplay = suggestions.slice(0, MAX_SUGGESTIONS);
  }

  if (!popup) {
      popup = document.createElement('div');
      popup.id = 'gif-suggestions-popup';
      document.body.appendChild(popup);
  }
  popup.style.visibility = 'hidden';
  popup.style.display = 'block';

  popup.innerHTML = '';
  currentSuggestionIndex = -1; // Reset index, ensures no highlight initially
  hideEmotePreview(); // Ensure preview is hidden when list is first shown/refreshed

  if (suggestionsToDisplay.length === 0) {
      if (popupExisted) { popup.style.display = 'none'; }
      // hideEmotePreview called above
      return;
  } else {
      const newHTML = suggestionsToDisplay.map((suggestionText, index) => {
          const key = suggestionText.substring(1);
          const imgSrc = localEmotelist[key];
          const isValidSrc = imgSrc && (imgSrc.startsWith('http:') || imgSrc.startsWith('https:') || imgSrc.startsWith('data:'));
          const imgTag = showEmotePreviews && isValidSrc ? `<img src="${imgSrc}" alt="${key}" class="suggestion-emote-preview" loading="lazy" />` : '';
          const displayValue = suggestionText; const insertValue = suggestionText;
          return `<div class="suggestion-item" data-index="${index}" data-insert-value="${insertValue}" data-display-value="${displayValue}" title="${insertValue}">${imgTag}<span class="suggestion-text">${displayValue}</span></div>`;
      }).join('');
      popup.innerHTML = newHTML;
  }

  // Positioning Logic (same as before)
  const targetRect = targetElement.getBoundingClientRect();
  const viewportHeight = window.innerHeight; const viewportWidth = window.innerWidth;
  const scrollY = window.scrollY; const scrollX = window.scrollX;
  const gap = 5;
  const popupHeight = popup.offsetHeight; const popupWidth = popup.offsetWidth;
  const spaceBelow = viewportHeight - targetRect.bottom - gap; const spaceAbove = targetRect.top - gap;
  let finalTop;
  if (spaceBelow >= popupHeight || spaceBelow >= spaceAbove) { finalTop = targetRect.bottom + scrollY + gap; }
  else if (spaceAbove >= popupHeight) { finalTop = targetRect.top + scrollY - popupHeight - gap; }
  else { finalTop = targetRect.bottom + scrollY + gap; }
  let finalLeft = targetRect.left + scrollX;
  popup.style.minWidth = `180px`;
  const maxPopupWidth = Math.min(Math.max(180, targetRect.width), viewportWidth - 20);
  popup.style.maxWidth = `${maxPopupWidth}px`;
   const currentPopupWidth = Math.min(popupWidth, maxPopupWidth);
  if (finalLeft + currentPopupWidth > (scrollX + viewportWidth - 10)) {
       finalLeft = scrollX + viewportWidth - currentPopupWidth - 10;
  }
  finalLeft = Math.max(scrollX + 10, finalLeft);
  popup.style.top = `${finalTop}px`; popup.style.left = `${finalLeft}px`;

  popup.style.visibility = 'visible';
  const suggestionElements = Array.from(popup.children);

  if (suggestionElements.length > 0) {
      attachSuggestionListeners(suggestionElements, targetElement); // Attaches mouse listeners
  } else {
      popup.style.display = 'none';
      // hideEmotePreview called earlier
  }

  // Re-attach outside click listener
  setTimeout(() => {
      document.removeEventListener('click', handleOutsideClick, true);
      document.addEventListener('click', handleOutsideClick, { capture: true, once: true });
  }, 50);
}


// --- MODIFIED: Attach Suggestion Item Listeners (Mouse Interaction) ---
function attachSuggestionListeners(suggestionElements, targetElement) {
    const popup = document.getElementById('gif-suggestions-popup');
    if (popup) {
        // Add listener to hide preview when mouse leaves the *entire* popup list
        popup._mouseleaveHandler = hideEmotePreview;
        popup.removeEventListener('mouseleave', popup._mouseleaveHandler); // Clean up old
        popup.addEventListener('mouseleave', popup._mouseleaveHandler);
    }

    suggestionElements.forEach((item, index) => {
        // --- Click handler (mostly unchanged, ensures popup closes) ---
        const oldClickListener = item._clickHandler;
        if (oldClickListener) { item.removeEventListener('click', oldClickListener); delete item._clickHandler; }
        const clickHandler = (e) => {
            e.preventDefault(); e.stopPropagation();
            const insertValue = item.getAttribute('data-insert-value');
            if (!insertValue || !document.body.contains(targetElement)) { removeSuggestionsPopup(); return; }
            const value = targetElement.value || "";
            let cursorPos = targetElement.selectionEnd ?? value.length;
            let wordStart = value.lastIndexOf(':', cursorPos - 1);
            let spaceBeforeColon = value.lastIndexOf(' ', wordStart -1);
            wordStart = (spaceBeforeColon === -1) ? 0 : spaceBeforeColon + 1;
            let actualColonPos = value.indexOf(':', wordStart);
            if (actualColonPos === -1 || actualColonPos >= cursorPos) { removeSuggestionsPopup(); return; }
            wordStart = actualColonPos;
            const newValue = value.substring(0, wordStart) + insertValue + " " + value.substring(cursorPos);
            targetElement.value = newValue;
            const newCursorPos = wordStart + insertValue.length + 1;
            if (typeof targetElement.selectionStart !== 'undefined') {
                targetElement.focus();
                requestAnimationFrame(() => {
                    try { targetElement.setSelectionRange(newCursorPos, newCursorPos); }
                    catch (err) { console.error("DEBUG: Error setting selection range:", err); }
                });
            }
            targetElement.dispatchEvent(new Event('input', { bubbles: true, cancelable: true }));
            removeSuggestionsPopup(); // Close popup (calls hideEmotePreview)
        };
        item._clickHandler = clickHandler;
        item.addEventListener('click', clickHandler);

        // --- Mouseover handler (Highlights item, triggering preview timer via highlightSuggestion) ---
        const oldMouseoverListener = item._mouseoverHandler;
        if(oldMouseoverListener) { item.removeEventListener('mouseover', oldMouseoverListener); delete item._mouseoverHandler; }
        const mouseoverHandler = () => {
            const itemIndex = parseInt(item.getAttribute('data-index'), 10);
            if (!isNaN(itemIndex) && currentSuggestionIndex !== itemIndex) {
                 // Update index state FIRST
                currentSuggestionIndex = itemIndex;
                // THEN call highlightSuggestion, which handles CSS and starts the timer
                highlightSuggestion(suggestionElements, currentSuggestionIndex);
            }
            // No need to directly manage timer here; highlightSuggestion does it.
        };
        item._mouseoverHandler = mouseoverHandler;
        item.addEventListener('mouseover', mouseoverHandler);

        // --- Mouseleave from *individual item* (No longer needed to manage timer) ---
         const oldMouseleaveListener = item._mouseleaveHandler;
         if(oldMouseleaveListener) { item.removeEventListener('mouseleave', oldMouseleaveListener); delete item._mouseleaveHandler; }
         // No action needed when leaving just one item, only when leaving the whole list (handled above).
    });
}

// --- Handle Clicking Outside the Popup ---
function handleOutsideClick(e) {
    const popup = document.getElementById('gif-suggestions-popup');
    const isEditableTargetRelated = e.target.closest('textarea[data-gif-suggest-listener], input[type="text"][data-gif-suggest-listener]');
    const isPopupOrChild = popup && popup.contains(e.target);
    const isEmotePreviewOrChild = emotePreviewPopupElement && emotePreviewPopupElement.contains(e.target); // Check new preview

    // Close if click is outside suggestions, outside preview, and outside related input
    if (popup && popup.style.display === 'block' && !isPopupOrChild && !isEmotePreviewOrChild && !isEditableTargetRelated) {
        removeSuggestionsPopup(); // Calls hideEmotePreview
    } else if (popup && popup.style.display === 'block') {
         // Click was inside relevant area, re-attach listener for next click
         document.removeEventListener('click', handleOutsideClick, true);
         setTimeout(() => {
            document.addEventListener('click', handleOutsideClick, { capture: true, once: true });
         }, 0);
    }
}

// --- MODIFIED: Remove Suggestions Popup (Calls hideEmotePreview) ---
function removeSuggestionsPopup() {
    const popup = document.getElementById('gif-suggestions-popup');
    if (popup) {
        if (popup.style.display !== 'none') {
            popup.style.display = 'none';
            popup.innerHTML = '';
            // Remove container listener for mouseleave if it exists
            if (popup._mouseleaveHandler) {
                popup.removeEventListener('mouseleave', popup._mouseleaveHandler);
                delete popup._mouseleaveHandler;
            }
        }
    }
    currentSuggestionIndex = -1; // Reset selection state
    document.removeEventListener('click', handleOutsideClick, true); // Remove outside click listener
    hideEmotePreview(); // Explicitly hide preview and clear timer
}